
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    print qq{1..0 # SKIP these tests are for testing by the author\n};
    exit
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Abhijit
Attachment
Byron
Client
DJ
Dean
Dmitri
Ellacott
Exception
Forms
Group
HTTPClient
Hamstead
Harrison
Lund
Marco
Menon
Object
Pessotto
Queue
REST
RT
Sarvesh
SearchResult
Stauffer
Søren
Ticket
Tikhonov
Tom
Transaction
User
ams
belg4mit
bobtfish
code
dean
dj
djzort
dkrotkine
dmitri
lib
melmothx
pplusdomain
sarveshd
soren
tomh
